import processing.opengl.*;
import javax.media.opengl.*;
import peasy.*;
import controlP5.*;

ControlP5 controlP5;
PMatrix3D currCameraMatrix;
PGraphics3D g3; 
ControlGroup l;
Slider mySlider2; 

Textlabel mySmallTextlabel;
Integrator slide; 

PeasyCam cam;
PGraphicsOpenGL pgl;
GL gl;

int nodeCount; 
Node[] nodes = new Node[100]; 
Node selection;
int edgeCount; 
Edge[] edges = new Edge[500];
HashMap nodeTable = new HashMap(); 

//Sketch Colors 
public int myColorRect = 200;

float cLength = 70; 
//static final color nodeColor = #F0C070; 
static final color nodeColor = #05B2FF; 
static final color fontColor = #aaaaaa; 
static final color selectColor = #FFAF2E; 
static final color fixedColor = #05DCFF; 
static final color edgeColor = #FFFFFF; 
color bgColor = color(0,0,0,0);


//Global Parameters: 
String node1; 
String node2; 
int maxCount = -1; 
int popular = 0;
int lastpopular = -1;
PFont font; 

PGraphics buffer;   // buffer for 3D picking...important!!!!

void setup() {
  size(1280,720,OPENGL); 
  hint(DISABLE_OPENGL_2X_SMOOTH); 
  hint(ENABLE_OPENGL_4X_SMOOTH); 
  hint(ENABLE_DEPTH_SORT); 

  font = loadFont("HelveticaNeue-58.vlw"); 

  //LoadData function: 
  loadData();
  
  //Camera Function: 
  cam = new PeasyCam(this, 300);
  cam.setMinimumDistance(50);
  cam.setMaximumDistance(6000);

  buffer = createGraphics(width, height, P3D);
  slide = new Integrator(700, 0.3, 0.8);
  
  setupGUI(); 

}
void draw() 
{    
  slide.update();   
  background(bgColor); 
  //this moves the interface up and down
  if(mouseY > 600)
  {
    slide.target(600); 
  }
  else
  {
   slide.target(700);  
  }
  if(l.position().y != slide.value)
  {
    l.position().y = slide.value; 
  }
  
  pgl = (PGraphicsOpenGL) g;
  gl = pgl.gl;
  pgl.beginGL();
 
  gl.glDepthMask(false); 
  gl.glEnable(GL.GL_DEPTH_TEST);
  gl.glClear(GL.GL_DEPTH_BUFFER_BIT);  
  gl.glEnable ( GL.GL_LINE_SMOOTH );
  
  gl.glEnable(GL.GL_BLEND);
  gl.glBlendFunc(GL.GL_SRC_ALPHA,GL.GL_ONE);
  
  gl.glEnable (GL.GL_LINE_SMOOTH);
  gl.glHint (GL.GL_LINE_SMOOTH_HINT, GL.GL_NICEST);
  
  for(int i = 0; i < edgeCount; i++)
  {
    edges[i].relax(); 
  }
  for(int i = 0; i < nodeCount; i++)
  {
    nodes[i].relax(); 
  }
  for(int i = 0; i < nodeCount; i++)
  {
    nodes[i].update(); 
    if(nodes[i].count > maxCount)
    {
      maxCount = nodes[i].count; 
      popular = i;
    }
  }
  for(int i = 0; i < edgeCount; i++)
  {
    edges[i].draw(); 
  }
  for(int i = 0; i < nodeCount; i++)
  {
    nodes[i].draw(this.g);
  } 
  
  if(popular != lastpopular)
  {
    nodes[popular].fixed = true; 
    nodes[popular].loc.x = 0; 
    nodes[popular].loc.y = 0; 
    nodes[popular].loc.z = 0; 
    if(lastpopular >=0)
    {
      nodes[lastpopular].fixed = false; 
    }
    lastpopular = popular; 
  } 
  
  stroke(200,50); 
  strokeWeight(2); 
  line(-100,0,0,100, 0,0); 
  line(0,-100,0,0, 100,0); 
  line(0,0,-100,0, 0,100); 
  
  pgl.endGL();
  gui();
}

void loadData()
{
  String[] lines = loadStrings("textfile.txt");
  //make the text into a single String Object 
  String sLine = join(lines, " "); 
  sLine = sLine.replaceAll("--","\u2014");
  //Split into phases using any of the provided tokens
  String[] phrases = splitTokens(sLine, ".,;:?!\u2014\"");
  for(int i = 0; i < phrases.length; i++)
  {
    //make this phrase lower case
    String phrase = phrases[i].toLowerCase(); 
    //Split each phrase into individual words at one or more spaces
    String[] words = splitTokens(phrase, " "); 
    for(int w = 0; w < words.length-1; w++)
    {
      addEdge(words[w],words[w+1]); 
    }
  }
//an example of how to add nodes with a connection
//  addEdge("visual", "map"); 

}

void addEdge(String fromLabel, String toLabel)
{
  Node from = findNode(fromLabel); 
  Node to = findNode(toLabel); 
  from.increment(); 
  to.increment(); 
  
  //Check to see whether this Edge already exists
  for(int i = 0; i < edgeCount; i++)
  {
    if(edges[i].from == from && edges[i].to == to)
    {
      edges[i].increment();
      return;
    }
  }
  
  Edge e = new Edge(from, to); 
  e.increment(); 
  if(edgeCount == edges.length)
  {
    edges = (Edge[]) expand(edges);
  }
  edges[edgeCount++] = e; 
}

Node findNode(String label)
{
  label = label.toLowerCase(); 
  Node n = (Node) nodeTable.get(label); 
  if(n == null)
  {
    return addNode(label);
  }
  return n; 
}

Node addNode(String label)
{
  Node n = new Node(label);
  if(nodeCount == nodes.length)
  {
    nodes = (Node[]) expand(nodes);
  }
  nodeTable.put(label,n); 
  nodes[nodeCount++] = n; 
  return n;
}

void mouseClicked()
{
  if (mouseEvent.getClickCount() == 2) 
  {
    buffer.beginDraw();
    buffer.background(getColor(-1)); // since background is not an object, its id is -1
    buffer.noStroke();
  
    buffer.setMatrix(((PGraphics3D)g).camera);
    
    for(int i = 0; i < nodeCount; i++)
    {
      nodes[i].drawBuffer(buffer);
    } 
    buffer.endDraw();
   
    // get the pixel color under the mouse
    color pick = buffer.get(mouseX, mouseY);
    // get object id
    int id = getId(pick);
    // if id > 0 (background id = -1)
    if (id >= 0) 
    {
      // change the cube color
      selection = nodes[id]; 
      nodes[id].focus(); 
    }
  } 
  if(mouseEvent.getClickCount() == 1)
  {
    selection = null; 
  }
}

color getColor(int id) {
  return -(id + 2);
}
 
// color -2 gives 0, etc.
int getId(color c) {
  return -(c + 2);
}

